#! /usr/bin/python3
# Last edited on 2021-09-04 02:39:56 by stolfi

import sys
import os
import numpy.random

def gendata(fname, pr, chop1, D1, chop2, D2):
  # Writes a file "{fname}" where each line has a random vector.
  # The vectors are generated by {toss(D1,chop1)} with probability {pr}
  # and {toss(D2,chop2)} with probability {1-pr}.
  wr = open(fname, "w");
  nt = 10000; # Number of observations.
  for it in range(nt):
    if it < pr*nt:
      X = toss(D1,chop1)
    else:
      X = toss(D2,chop2)
    for v in X:
      wr.write(" %+12.6f" % v)
    wr.write("\n")
  wr.close()
  # ----------------------------------------------------------------------
 
def toss(D,chop):
  # The parmeter {D} must be a pair {(a,M)} where {a} is a list of {nc}
  # floats and {M} is a matrix (list of lists) of {nc} by {nc} floats.
  # If {chop} is false, the procedure generates a vector of {nc} normal
  # deviates, then post-multiplies it by {M}, adds {a}, and returns the
  # result. If {chop} is true, the initial deviates are restricted to those 
  # with positive first coordinate.
  a, M = D
  nc = len(a)
  r = [ toss1(chop and ic==0) for ic in range(nc) ]
  s = list(a).copy()
  for jc in range(nc):
    for ic in range(nc):
      s[jc] += r[ic]*M[ic][jc]
  return s
  # ----------------------------------------------------------------------
  
def toss1(chop):
  # If {chop} is false, returns a normal deviate.
  # If {chop} is true, returns a random number whose distr is the positive half of 
  # the normal distr.
  x = numpy.random.normal(0,1)
  while chop and x < 0:
    x = numpy.random.normal(0,1)
  return x
  # ----------------------------------------------------------------------

gendata("rndA.txt", 0.80, False, ((0,0), ((1,0),(0,1))),  False, ((0,0), ((9,0),(0,9))))              # Two round clouds of different sizes.
gendata("rndB.txt", 0.80, False, ((0,0), ((1,0),(0,1))),  False, ((3,5), ((1,0),(0,1))))              # Two round clouds of same size and different means.
gendata("rndC.txt", 0.80, False, ((0,0), ((1,0),(0,1))),  False, ((0,0), ((+9.0,+3.0),(-0.1,+0.3))))  # A round cloud and a diagonal cloud, same mean.
gendata("rndD.txt", 0.80, True,  ((0,0), ((1,0),(0,1))),  True,  ((0,0), ((-1.0,00.0),(00.0,+0.3))))  # A half-round cloud and a half-elliptical cloud.
gendata("rndE.txt", 0.80, True,  ((0,0), ((9,0),(0,1))),  True,  ((0,0), ((0,9),(1,0))))              # An elongated cloud bent 90 degrees.
